
function [prob,Gs]=cmccvs(stat,lambda,sigma,statIndex,plotIndex,T,nReps);
%
% simulates the Clark-McCracken distributions for the MSE-T and ENC-T
% statistics (see more in Clark-McCracken (2002), Evaluating Long-Horizon Forecasts)
% statIndex  1    DM stat
%            2    t stat
defT=100; defnReps=1e4;
switch nargin;
   case 4; plotIndex=0; T=defT; nReps=defnReps; 
   case 5; T=defT; nReps=defnReps; 
   case 6; nReps=defnReps; 
end;
initSeed; 
k=size(sigma,1); P=chol(sigma); lambda=lambda-mod(lambda,1/T); %make sure the grid has the right size
gridG=(lambda:(1-lambda)/T:1)'; dt=repmat(1/T,T+1,1); G=repmat(nan,nReps,1); GT=G;

for i=1:nReps;
   w1=cumsum([sqrt(lambda)*randn(1,k); randn(T+1,k)/sqrt(T)]*P);
   w2=w1(1:end-1,:);
   dw2=diff(w1);
   if k==1; G1=sum((w2.*dw2)./gridG); G2=sum((w2.*w2).*( dt./(gridG.^2) ) ); G3=sum(diag(w2*sigma*w2').*(dt./(gridG.^2) ) );
   else G1=sum(sum((w2.*dw2)')'./gridG); G2=sum(sum((w2.*w2)')'.*( dt./(gridG.^2) ) ); G3=sum(diag(w2*sigma*w2').*(dt./(gridG.^2) ) ); end;
   G(i)=(G1-.5*G2)/sqrt(G3);
   GT(i)=G1/sqrt(G3);
end;
if statIndex==1; Gs=sort(G); titleString='t Statistic'; elseif statIndex==2; Gs=sort(GT); titleString=('DM Statistic'); end; 
fprintf('Mean of Clark-McCracken Statistic:  %3.3f\n',mean(Gs));
gProb=(1:nReps)'/nReps;
prob=repmat(nan,length(stat),1); 
prob = interp1(Gs,gProb,stat,'linear'); p2=interp1(Gs,gProb,0,'linear'); fprintf('Prob of stat<0:  %5.5f\n',p2);
gMed=Gs(round(nReps/2)); prob(isnan(prob)&(stat>gMed))=1;  prob(isnan(prob)&(stat<gMed))=0;
g95=Gs(round(.95*nReps)); g90=Gs(round(.9*nReps));
if plotIndex; 
	[f,xi]=ksdensity(Gs);
	mp1=figure; myPlot(xi,[f' normpdf(xi)]); legend('Clark-McCracken','Standard Normal'); title(titleString);
    xlabel(['k_2 = ' int2str(length(sigma)) ', \pi = ' num2str((1-lambda)/lambda) ', mean = ' num2str(mean(Gs)) ' , median = ' num2str(gMed) ', ' int2str(nReps) ' reps']);
    mp2=figure; myPlot(Gs,[gProb normcdf(Gs)]); legend('Clark-McCracken','Standard Normal',2); title(['Clark-McCracken cdf, ' titleString]);
    xlabel(['k_2 = ' int2str(length(sigma)) ', \pi = ' num2str((1-lambda)/lambda) ', mean = ' num2str(mean(Gs)) ' , median = ' num2str(gMed) ', ' int2str(nReps) ' reps']);
    fprintf('Clark-McCracken Statistics:  %3.3f   %3.3f  P values: %5.5f %5.5f\n',stat(1),stat(2),1-prob(1),1-prob(2));
    input('Continue? [Hit Enter.]'); close([mp1 mp2]);
 end;
