function [dmS,sd,pVal]=dmCM(errorsSmall,errorsBig,xSmall,xBig,lambda,nLags,plotIndex,nT,nReps);
% Calculates the Diebold-Mariano test based on OLS and NW HAC standard
% errors (all tests compare the forecasts wrt the last forecast)
% inputs   xTilde         regressors in the big model not present in the
% small plus a constant
%         nLags           number of lags for standard errors
%         
% outputs     dmS         k x 2 matrix of D-M statistics, 1st column OLS stats, 2nd column HAC stats
%             sd          k x 2 matrix of standard errors
%             pVal        k x 2 matrix of p values (based on normal distribution)
% 
if nargin==6; plotIndex=0; nT=200; nReps=10000; end;
if nargin==7; nT=200; nReps=10000; end;
if nargin ==8; nReps=10000; end;
%nReps=1e2;
T=length(errorsSmall); errors2=[errorsSmall.^2 errorsBig.^2];
dmS=repmat(nan,1,2); sd=dmS; pVal=dmS;

diffErrs2=errors2(:,1)-errors2(:,2); 
sd(1)=sqrt((var(diffErrs2))/T); 
if nLags >0;
   sd0=hac(diffErrs2,nLags,0,3)/T; sd(2)=sqrt(sd0); 
else sd(2)=sd(1); 
end;
if sd(1)>0; dmS(1)=mean(diffErrs2)/sd(1); dmS(2)=mean(diffErrs2)/sd(2); 
elseif sd(1)==0 & diffErrs2(i)==0; dmS=zeros(1,2); 
end;

k=size(xBig,2); k1=size(xSmall,2); fprintf('k_2:   %2.0f\n',k-k1);
h2=repmat(errorsSmall,1,k).*xBig;
xbb=(repmat(xBig,[1,1,k])); xbbp=permute(xbb,[1 3 2]); b2=inv(squeeze(mean(xbb.*xbbp,1)));
xss=(repmat(xSmall,[1,1,k1])); xssp=permute(xss,[1 3 2]); b1=inv(squeeze(mean(xss.*xssp,1)));
J=[eye(k1),zeros(k1,k-k1)];  
b2m1=inv(sqrtm(b2)); aTildeA=b2m1*(-J'*b1*J+b2)*b2m1;
% the decomposition below workd better than this raTildeA=myRound(aTildeA,8); [u,ev]=chol(raTildeA); J2=[zeros(k-k1,k1), eye(k-k1)]; % this rounding seems to be necessary because matlab sometimes produces complex eigenvalues
tol=1e-6;
[u,ev]=schur(aTildeA); probEvs=sum(sum((abs(ev)>tol&abs(1-ev)>tol))); 
if probEvs>0; disp(['Check the spectral decomposition in ' mfilename ', some eigenvalues are not correctly calculated.']); 
    disp('Eigenvalue Matrix'); disp(ev); 
end;
indAA=logical(max((abs(ev-1)<tol)));
if sum(indAA)~=(k-k1); error('The spectral decomposition of A~''A~ is not precise, increase tol in dmCM'); end;
J2=ev(indAA,:); J2(abs(J2)<tol)=0; J2(abs(J2-1)<tol)=1; aTilde=J2*(u');
hTilde=(inv(sqrt(var(errorsSmall)))*aTilde*sqrtm(b2)*h2')';
hTildeVar=hac(hTilde,nLags,0,3);
pVal=1-cmccvs3(dmS,lambda,hTildeVar,1,plotIndex,nT,nReps);
