function [dmS,sd,pVal]=dm(errors2,nLags);
% Calculates the Diebold-Mariano test based on OLS and NW HAC standard
% errors (all tests compare the forecasts wrt the last forecast)
% inputs   forecasts      matrix of k forecasts
%         actual          vector of actual values
%         nLags           number of lags for standard errors
%         
% outputs    dmS           k x 2 matrix of D-M statistics, 1st column OLS stats, 2nd column HAC stats
%             sd          k x 2 matrix of standard errors
%             pVal        k x 2 matrix of p values (based on normal distribution)
% 
[T,k]=size(errors2);
dmS=repmat(nan,k,2); sd=dmS; pVal=dmS; 
for i=1:k; 
   diffErrs2(:,i)=errors2(:,k)-errors2(:,i); 
   sd(i,1)=sqrt((var(diffErrs2(:,i)))/T); 
   if nLags >0;
        sd0=hac(diffErrs2(:,i),nLags,0,3)/T; sd(i,2)=sqrt(sd0); 
   else sd(i,2)=sd(i,1); 
   end;
   if sd(i,1)>0; dmS(i,1)=mean(diffErrs2(:,i))/sd(i,1); dmS(i,2)=mean(diffErrs2(:,i))/(sd(i,2)); 
   elseif sd(i,1)==0 & diffErrs2(:,i)==0; dmS(i,:)=zeros(1,2); 
   end;
end;
pVal=2*(1-normcdf(abs(dmS)));