end clear memory
env noecho
cal 1978 1 12

dis %dateandtime()
allocate 2003:12 
open data C:\sentiment\matlab\data\SENTIMENTDATA78.CSV
data(for=cdf,org=col)

*=============================================================
*  Choose Parameters
*=============================================================
comp lags = 2;			* # of lags on the RHS
comp lagsS =1;
comp hForward = 3;		* periods ahead for forecasting
comp consumptionIndex=1;	* choose consumption series
comp sentimentIndex = 1; 	* choose sentiment series		
comp startReg = 1978:1;	
comp splitReg =1993:12;		* where to split the two regressions
comp endReg = 2002:12;
comp rob =1;			* robust errors?
comp printI =0;
comp sentStart =0;		* which sentiment lags to include?
*=============================================================
* renaming the series
smpl startReg endReg 
set totMich = hhsntr
set expMich = hhsntn
set totCB = hhcnf
set expCB = hhcnxp
set fed_funds = fyff
set sp = fspcom
set dispInc = gmyd
set totCons = gmcq
set durCons = gmcdq
set nondurCons = gmcnq
set servCons = gmcsq
set cpi = punew
set un = lhur
set coinc = dcoinc
set leadd = dlead
set rSpread = fyaaac-fygm3
set lsSpread = fygt10-fygm3

* some transformations
set dfed_funds = fed_funds - fed_funds{1}
set infl = 1200*( log(cpi) - log(cpi{1}) )
set totConsG = 1200*( log(totCons) - log(totCons{1}) )
set durConsG = 1200*( log(durCons) - log(durCons{1}) )
set nondurConsG = 1200*( log(nondurCons) - log(nondurCons{1}) )
set servConsG = 1200*( log(servCons) - log(servCons{1}) )
* not that the growth rates below are ahead, ie cons growth in jan 2001 is tg over the next year or so

* LHS
set totConsLHS = 1200*( log(totCons{-hForward+1})-log(totCons{1})  )/hForward
set durConsLHS = 1200*( log(durCons{-hForward+1})-log(durCons{1}) )/hForward
set nondurConsLHS = 1200*( log(nondurCons{-hForward+1})-log(nondurCons{1}) )/hForward
set servConsLHS = 1200*( log(servCons{-hForward+1})-log(servCons{1}) )/hForward

if consumptionIndex==1
	{
	set consumption = totConsG
	set consumptionLHS = totConsLHS
	}
else if consumptionIndex==2
	{
	set consumption = durConsG
	set consumptionLHS = durConsLHS
	}
else if consumptionIndex==3
	{
	set consumption = nondurConsG
	set consumptionLHS = nondurConsLHS
	}
else if consumptionIndex==4
	{
	set consumption = servConsG
	set consumptionLHS = servConsLHS
	}
end if

**************************************************************************************
*  TABLE 1 ALTERNATIVE REGRESSIONS
**************************************************************************************
source myTabAdjR2.src

smpl startReg endReg
declare vector[integer] allVars
enter(varying) allVars
# totConsG{1 to lags} fyff{0 to lags} infl{1 to lags} un{1 to lags} lsSpread{0 to lags}

disp 'WHOLE SAMPLE'

disp '   Big         	   Non-sentiment'
@myTabAdjR2(nLags=hForward-1,robust=rob,printout=printI) totConsLHS
#allVars
#totConsG{1 to lags}
#totMich{sentStart to lagsS} 
#expMich{sentStart to lagsS} 
#totCB{sentStart to lagsS} 
#expCB{sentStart to lagsS} 

*====================================================================================

smpl startReg splitReg
disp '1ST SUB-SAMPLE'

disp '   Big         	   Non-sentiment'
@myTabAdjR2(nLags=hForward-1,robust=rob,printout=printI) totConsLHS
#allVars
#totConsG{1 to lags}
#totMich{sentStart to lagsS} 
#expMich{sentStart to lagsS} 
#totCB{sentStart to lagsS} 
#expCB{sentStart to lagsS} 


*=====================================================================================

smpl splitReg+1 endReg
disp '2ND SUB-SAMPLE'

disp '   Big         	   Non-sentiment'
@myTabAdjR2(nLags=hForward-1,robust=rob,printout=printI) totConsLHS
#allVars
#totConsG{1 to lags}
#totMich{sentStart to lagsS} 
#expMich{sentStart to lagsS} 
#totCB{sentStart to lagsS} 
#expCB{sentStart to lagsS} 



*=====================================================================================

smpl startReg endReg
disp ''; disp '========================================================================'
disp 'WHOLE SAMPLE'

stats totConsLHS

smpl startReg splitReg
disp '1ST SUB-SAMPLE'

stats totConsLHS

smpl splitReg+1 endReg
disp '2ND SUB-SAMPLE'

stats totConsLHS


*=====================================================================================
disp 'Adj R^2s'
smpl startReg endReg

linreg(noprint) totConsLHS
# constant totConsG{1 to lags} fyff{0 to lags} infl{1 to lags} un{1 to lags} lsSpread{0 to lags} totMich{0 to lags} 
comp Rall = %rbarsq

linreg(noprint) totConsLHS
# constant fyff{0 to lags} infl{1 to lags} un{1 to lags} lsSpread{0 to lags} totMich{0 to lags} 
comp Rcons = %rbarsq

linreg(noprint) totConsLHS
# constant totConsG{1 to lags} infl{1 to lags} un{1 to lags} lsSpread{0 to lags} totMich{0 to lags} 
comp Rfyff = %rbarsq

linreg(noprint) totConsLHS
# constant totConsG{1 to lags} fyff{0 to lags} un{1 to lags} lsSpread{0 to lags} totMich{0 to lags} 
comp Rinfl = %rbarsq

linreg(noprint) totConsLHS
# constant totConsG{1 to lags} fyff{0 to lags} infl{1 to lags} lsSpread{0 to lags} totMich{0 to lags} 
comp Run = %rbarsq

linreg(noprint) totConsLHS
# constant totConsG{1 to lags} fyff{0 to lags} infl{1 to lags} un{1 to lags} totMich{0 to lags} 
comp Rs = %rbarsq

linreg(noprint) totConsLHS
# constant totConsG{1 to lags} fyff{0 to lags} infl{1 to lags} un{1 to lags} lsSpread{0 to lags}
comp Rsent = %rbarsq

disp @3 'all' @13 'cons' @23 'fyff' @33 'infl' @43 'un' @53 'spread' @63 'totMich'
disp @3 #.### Rall @13 #.### Rcons @23 #.### Rfyff @33 #.### Rinfl @43 #.### Run @53 #.### Rs @63 #.### Rsent

linreg(noprint) totConsLHS
# constant totConsG{1 to lags} fyff{0 to lags} infl{1 to lags} un{1 to lags} lsSpread{0 to lags} expMich{0 to lags} 
comp Rall = %rbarsq

linreg(noprint) totConsLHS
# constant fyff{0 to lags} infl{1 to lags} un{1 to lags} lsSpread{0 to lags} expMich{0 to lags} 
comp Rcons = %rbarsq

linreg(noprint) totConsLHS
# constant totConsG{1 to lags} infl{1 to lags} un{1 to lags} lsSpread{0 to lags} expMich{0 to lags} 
comp Rfyff = %rbarsq

linreg(noprint) totConsLHS
# constant totConsG{1 to lags} fyff{0 to lags} un{1 to lags} lsSpread{0 to lags} expMich{0 to lags} 
comp Rinfl = %rbarsq

linreg(noprint) totConsLHS
# constant totConsG{1 to lags} fyff{0 to lags} infl{1 to lags} lsSpread{0 to lags} expMich{0 to lags} 
comp Run = %rbarsq

linreg(noprint) totConsLHS
# constant totConsG{1 to lags} fyff{0 to lags} infl{1 to lags} un{1 to lags} expMich{0 to lags} 
comp Rs = %rbarsq

linreg(noprint) totConsLHS
# constant totConsG{1 to lags} fyff{0 to lags} infl{1 to lags} un{1 to lags} lsSpread{0 to lags} 
comp Rsent = %rbarsq

disp @3 'all' @13 'cons' @23 'fyff' @33 'infl' @43 'un' @53 'spread' @63 'expMich'
disp @3 #.### Rall @13 #.### Rcons @23 #.### Rfyff @33 #.### Rinfl @43 #.### Run @53 #.### Rs @63 #.### Rsent

linreg(noprint) totConsLHS
# constant totConsG{1 to lags} fyff{0 to lags} infl{1 to lags} un{1 to lags} lsSpread{0 to lags} totCB{0 to lags} 
comp Rall = %rbarsq

linreg(noprint) totConsLHS
# constant fyff{0 to lags} infl{1 to lags} un{1 to lags} lsSpread{0 to lags} totCB{0 to lags} 
comp Rcons = %rbarsq

linreg(noprint) totConsLHS
# constant totConsG{1 to lags} infl{1 to lags} un{1 to lags} lsSpread{0 to lags} totCB{0 to lags} 
comp Rfyff = %rbarsq

linreg(noprint) totConsLHS
# constant totConsG{1 to lags} fyff{0 to lags} un{1 to lags} lsSpread{0 to lags} totCB{0 to lags} 
comp Rinfl = %rbarsq

linreg(noprint) totConsLHS
# constant totConsG{1 to lags} fyff{0 to lags} infl{1 to lags} lsSpread{0 to lags} totCB{0 to lags} 
comp Run = %rbarsq

linreg(noprint) totConsLHS
# constant totConsG{1 to lags} fyff{0 to lags} infl{1 to lags} un{1 to lags} totCB{0 to lags} 
comp Rs = %rbarsq

linreg(noprint) totConsLHS
# constant totConsG{1 to lags} fyff{0 to lags} infl{1 to lags} un{1 to lags} lsSpread{0 to lags}
comp Rsent = %rbarsq

disp @3 'all' @13 'cons' @23 'fyff' @33 'infl' @43 'un' @53 'spread' @63 'totCB'
disp @3 #.### Rall @13 #.### Rcons @23 #.### Rfyff @33 #.### Rinfl @43 #.### Run @53 #.### Rs @63 #.### Rsent

linreg(noprint) totConsLHS
# constant totConsG{1 to lags} fyff{0 to lags} infl{1 to lags} un{1 to lags} lsSpread{0 to lags} expCB{0 to lags} 
comp Rall = %rbarsq

linreg(noprint) totConsLHS
# constant fyff{0 to lags} infl{1 to lags} un{1 to lags} lsSpread{0 to lags} expCB{0 to lags} 
comp Rcons = %rbarsq

linreg(noprint) totConsLHS
# constant totConsG{1 to lags} infl{1 to lags} un{1 to lags} lsSpread{0 to lags} expCB{0 to lags} 
comp Rfyff = %rbarsq

linreg(noprint) totConsLHS
# constant totConsG{1 to lags} fyff{0 to lags} un{1 to lags} lsSpread{0 to lags} expCB{0 to lags} 
comp Rinfl = %rbarsq

linreg(noprint) totConsLHS
# constant totConsG{1 to lags} fyff{0 to lags} infl{1 to lags} lsSpread{0 to lags} expCB{0 to lags} 
comp Run = %rbarsq

linreg(noprint) totConsLHS
# constant totConsG{1 to lags} fyff{0 to lags} infl{1 to lags} un{1 to lags} expCB{0 to lags} 
comp Rs = %rbarsq

linreg(noprint) totConsLHS
# constant totConsG{1 to lags} fyff{0 to lags} infl{1 to lags} un{1 to lags} lsSpread{0 to lags}
comp Rsent = %rbarsq

disp @3 'all' @13 'cons' @23 'fyff' @33 'infl' @43 'un' @53 'spread' @63 'expCB'
disp @3 #.### Rall @13 #.### Rcons @23 #.### Rfyff @33 #.### Rinfl @43 #.### Run @53 #.### Rs @63 #.### Rsent

* single regressors
linreg(noprint) totConsLHS
# constant totConsG{1 to lags}
comp Rcons = %rbarsq

linreg(noprint) totConsLHS
# constant fyff{0 to lags}
comp Rfyff = %rbarsq

linreg(noprint) totConsLHS
# constant infl{1 to lags}
comp Rinfl = %rbarsq

linreg(noprint) totConsLHS
# constant un{1 to lags} 
comp Run = %rbarsq

linreg(noprint) totConsLHS
# constant lsSpread{0 to lags}
comp Rs = %rbarsq

linreg(noprint) totConsLHS
# constant totMich{0 to lags} 
comp RtotMich = %rbarsq

linreg(noprint) totConsLHS
# constant expMich{0 to lags} 
comp RexpMich = %rbarsq

linreg(noprint) totConsLHS
# constant totCB{0 to lags} 
comp RtotCB = %rbarsq

linreg(noprint) totConsLHS
# constant expCB{0 to lags} 
comp RexpCB = %rbarsq

disp 'Single Regressors'
disp @13 'cons' @23 'fyff' @33 'infl' @43 'un' @53 'spread' @63 'totMich' @73 'expMich' @83 'totCB' @93 'expCB' 
disp @13 #.### Rcons @23 #.### Rfyff @33 #.### Rinfl @43 #.### Run @53 #.### Rs @63 #.### RtotMich @73 #.### RexpMich @83 #.### RtotCB @93 #.### RexpCB
