function [minCr,lagsInd,infoC] = infoCrit(Xmax,y,ind,minLag);
%=======================================================================
% y = infoCrit(Xmax,y,ind,minLag);
% input:
% calculates AIC, BIC and Hannan-Quinn info Criteria (after Stock's aicbic)
% Xmax    MAXIMAL design matrix
% y       dependent variable
% ind     m x 2 matrix identifying the position of indep vars
% minLag  m x 1 vector of minimum # of lags for each variable
%
% output
% lagsInd  3 x cols(Xmax)  matrix of minimum indexes
% minCr    1 x 3 vector of minimum values of info crits  
% infoC    . x 3 matrix of all values
% Note: the value functions are T*(what some other programs give)     
%       also note that T is constant across tested models (as it should be)
%       jirka 11/21/02
%
%=======================================================================
if any(minLag<0); error('Minimum number of lags has to be non-negative.'); end;

ind=sortrows(ind,1); varSpread=ind(:,2)-ind(:,1)+1; T=size(Xmax,1);
sMax=tril(ones(max(varSpread)+1),-1); sMax=sMax(:,1:end-1); 
s1=sMax( (minLag(1)+1):(varSpread+1), 1:varSpread(1) ); % creates the maximal index matrix

for i=2:size(ind,1);  %create the big index matrix
   si=sMax(minLag(i)+1:varSpread(i)+1, 1:varSpread(i));
   s = [ kron(s1,ones(size(si,1),1)) kron(ones(size(s1,1),1),si)]; s1=s;
end; s=s1;
s1=ones(size(s,1),size(Xmax,2)); j=1;
for i=1:size(ind,1); % put in ones for the rest of indexes (vars not included in IC, ie supply shocks)
   s1(:,ind(i,1):ind(i,2))=s(:,j:(j+varSpread(i)-1));
   j=j+varSpread(i);
end; 
s=s1; aicPen=2/T; bicPen = log(T)/T; hqPen = log(log(T))/T; lssr=repmat(nan,size(s,1),1);

for i=1:size(s,1);
   Xi=Xmax(:,logical(s(i,:))); 
   beta=pinv(Xi'*Xi)*Xi'*y; e=(y-Xi*beta); lssr(i)=log((e'*e)/T);
   infoC(i,:) = lssr(i) + size(Xi,2)*[aicPen bicPen hqPen];
end;
[minCr,posCr]=min(infoC); indCr=s(posCr',:);
for i=1:size(ind,1);
   if max(size(ind(i,1):ind(i,2)))==1; lagsInd(:,i)=indCr(:,ind(i,1):ind(i,2));
   else; lagsInd(:,i)=sum(indCr(:,ind(i,1):ind(i,2))')'; end;
end;
