function [ftstat,ftname]=regprint(bhat,xname,xlag,vcv,e,n1,n2,rsq,constIndex);
%--------------------------------------------------------------------------
%         regprint.m     (originally after stock's regprint.prc)   jirka 10/17/02
%--------------------------------------------------------------------------
%   [ftstat,ftname]=regprint(bhat,xname,xlag,vcv,e,n1,n2,rsq);
%   regression printout - single equation OLS regression
%   uses le sage's selif.m function
%
if nargin==8; constIndex=1; end;
k=length(bhat);
if isempty(xlag); xlag=zeros(k,1); end;
if size(xname,1)~=k; xname=[]; for i=1:k; xname=strvcat(xname,'NA'); end; end;
if iscell(xname)~=1; xname=cellstr(xname); end;
if size(bhat,2)>size(bhat,1); bhat=bhat'; end;
if size(xlag,2)>size(xlag,1); xlag=xlag'; end;
labelLength=8;  %length of the printout labels (important for nice formatting)
xnamet={}; ftname={}; ftstat=[]; nobs=n2-n1+1; k=length(bhat); j=0; 
ssr=sum(e.^2); see=sqrt(ssr/(nobs-k)); sehat=sqrt(diag(vcv)); 
if constIndex;
	fst=rsq(1)/(1-rsq(1))*(nobs-k)/(k-1);
	if exist('fcdf'); fstProb=1-fcdf(fst,k-1,nobs-k); else; fstProb=repmat(nan,size(fst)); end;
	fstRob=bhat(2:end)'*pinv(vcv(2:end,2:end))*(bhat(2:end)); %/(k-1);  %this is a Chi2 test, for F test divide by dgs of fr
	if exist('chi2cdf'); fstRobProb=1-chi2cdf(fstRob,k-1); else; fstRobProb=repmat(nan,size(fstRob)); end;
end;

dw=sum(diff(e).^2)/sum(e.^2);
lbSpan=18;%size(bhat,1);  %default for Ljung-Box Q window
acWeights=(nobs-1:-1:nobs-lbSpan)';
lbQ=nobs*(nobs+2)*sum((acf(e,lbSpan,0).^2)./acWeights);
if exist('chi2cdf'); lbQprob=nan; else; lbQprob=nan; end;
fprintf('\n');
fprintf('=============================================================================\n');
fprintf('From Observation %4.0f TO%4.0f\n',n1,n2);
fprintf('Total Observations:%4.0f       Degrees of Freedom:%4.0f\n',nobs,nobs-k);
fprintf('(Cent) R**2:          %4.3f   R bar**2:             %4.3f\n',rsq(1),rsq(2));
fprintf('SSR:                 %4.3f   SEE:                  %4.3f     var resid:   %4.3f\n',ssr,see,var(e));
if constIndex; fprintf('F Stat(%s) LS:     %4.3f   Significance (F):   %7.7f\n',strcat(int2str(k-1),',',int2str(nobs-k)),fst,fstProb);
fprintf('F Stat(%s) NW:    %4.3f   Significance (Ch2):   %7.7f\n',strcat(int2str(k-1),',',int2str(nobs-k)),fstRob,fstRobProb); end;
fprintf('Durbin-Watson Stat:   %4.3f   Ljung-Box Q Stat:     %4.3f  LB sig: %5.5f\n',dw,lbQ,lbQprob);
fprintf('First Autocorrelation of Residuals:     %4.3f\n\n',acf(e,1,0));
fprintf('NO.     LABEL     LAG  COEFFICIENT  STAND. ERROR  t-STATISTIC  SIGNIF (Norm)\n');
fprintf('***    *******    ***  ***********  ************  ***********  *************\n');
for i=1:k;
   if bhat(i)>=0; bhatspace=' '; else bhatspace=''; end;
   tstat=bhat(i)/sehat(i);
   if exist('normcdf'); pval=2*(1-normcdf(abs(tstat))); else; pval=repmat(nan,1,1); end;
   fprintf('%3.0f     %s  %2.0f  %s  %5.5f        %4.3f     %s  %4.3f        %5.5f\n',i,...
   blank(xname{i},labelLength),xlag(i),bhatspace,bhat(i),sehat(i),bhatspace,tstat,pval);  
end;
fprintf('\n');
for i=1:(k-1);
    r0=(strcmp(xname{i},xname));
    if sum(r0)>1; j=j+1;   % do you want to run an F test? 
        if j==1;
            disp('             F-Tests on Blocks of Lags (NW)');
            disp('           VARIABLE      F-STATISTIC      SIGNIF. LEVEL (CHI2)');
        end;
        xnamet={xnamet{:},xname{i}};
        if sum(strcmp(xname{i},xnamet))==1; % checking for multiple names in the block
            ftname={ftname{:},xname{i}};
            r=diag(r0);
            r=selif(r,sum(r~=zeros(k)));
            f=(r*bhat)'*inv(r*vcv*(r'))*(r*bhat);%/size(r,1);  %this is a Chi2 test, for F test divide by dgs of fr
            if exist('chi2cdf');  signif=1-chi2cdf(f,size(r,1)); else; signif=nan; end;
            ftstat=[ftstat; [f signif]];
            fprintf('            %s        %4.3f                  %7.7f\n', blank(xname{i},labelLength),f,signif);
        end;             % sum if   
    end; %if sumr0
end;  % i 
fprintf('=============================================================================\n\n');
ftname=reshape(ftname,length(ftname),1);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%  utility function for regprint, makes sure given string has exactly k
%  characters, either cuts off or adds spaces
%
function y=blank(x,k);
if ischar(x)~=1; error('Function blank: First input has to be a string.'); end;
[nRows,nCols]=size(x);
if nCols>=k; y=x(:,1:k); 
else y=cat(2,x,repmat(' ',nRows,k-nCols)); end;
   
