function [mse,lambda,seLambda,pValDM,pValMZ,bfc,sds,dmStats,pValsDMCM,pValsDMT]=sentimentForecastFunc(consIn, sentIn,hF);
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
disp('==================================================================================');
initSeed;
load ./data/sentimentdata78.mat;
cc=customColors;

% Choose parameters
%==========================================================================
rollSpan=5;         %rollSpan: # years for window (at the beginning)
fixWindow=0;        % fixWindow: 1 fixed, otherwise increasing
lags=2;             %# of lags in the forecasting regressions % be careful, with 4 lags it takes about 9 min on Pentium 1.80, with 6 lags several hours
pMax=lags;          % maximum number of lags for information criteria
consIndex=consIn;        % consumption index: 1 total, 2 durable, 3 nondurable, 4 services
sentIndex=sentIn;        % sentiment index: 1 michigan total, 2 michigan expectations, 3 CB total, 4 CB expectations                         
smooth=36;          % smoothin' the MSEs
bootReps=50;        % # of bootstrap repetitions
hForward=hF;         % # of periods ahead to calculate forecasts (ie 12: forecasting consumption growth over next year
consIncIndex=1;         % include lagged consumption growth in the models?
fixIndex=1;         % determines the number of lags: 1 equal to lags, 2 using info criteria
bicIndex=1;         % BIC or AIC ?
plsIndex=0;         % predictive least squares?
printIndex=0;       % print out regression diagnostics for forecasts
mseRollPrint=0;     % print the rolling MSE graph

% Regresion Matrices
%==========================================================================
transformData; %call script to rename and 'stationarize' data
rollEnd=2002; rollStart=1978+rollSpan; %rollStart=1987; % uncomment to pick rollStart by yourself
fprintf('Time Frame:  %4.0f  %4.0f      Lags: %2.0f\n', rollStart,rollEnd,lags);
up=floor(.975*bootReps); mid=round(.5*bootReps); down=ceil(.025*bootReps); 

switch consIndex; 
    case 1; consumption=totalCons; consumptionLHS=totalConsLHS; seriesNames='Total Consumption      '; 
    case 2; consumption=durableCons; consumptionLHS=durableConsLHS; seriesNames='Durable Consumption        ';
    case 3; consumption=nondurableCons; consumptionLHS=nondurableConsLHS; seriesNames='Nondurable Consumption       '; 
    case 4; consumption=serviceCons; consumptionLHS=serviceConsLHS; seriesNames='Services       '; 
end;
switch sentIndex; 
    case 1; sentiment=michSent; sentName=' Total Michigan'; seriesNames=[seriesNames '   Total Michigan']; 
    case 2; sentiment=michExp; sentName='Michigan Expectations'; seriesNames=[seriesNames '   Michigan Expectations'];
    case 3; sentiment=cbSent; sentName='Total CB'; seriesNames=[seriesNames '    Total CB']; 
    case 4; sentiment=cbExp; sentName='CB Expecations'; seriesNames=[seriesNames ' CB Expectations']; 
end;
disp(seriesNames);

if consIncIndex==1; X=mklagsMine([consumption sentiment],[1 0; lags lags-1]); else X=mklagsMine([sentiment],[0; lags-1]); end;    
XsentOnly=[ones(length(X.xl),1) X.xl];
X=mklagsMine([consumption],[1; lags]); XCons=[ones(length(X.xl),1) X.xl];
if consIncIndex==1; X=mklagsMine([consumption fyff inflation un lsSpread],[1 0 1 1 0; lags lags-1 lags lags lags-1]); 
else X=mklagsMine([fyff inflation un lsSpread],[0 1 1 0; lags-1 lags lags lags-1]); end;
XnonsentOnly=[ones(length(X.xl),1) X.xl];
if consIncIndex==1; X=mklagsMine([consumption sentiment fyff inflation un lsSpread],[1 0 0 1 1 0; lags lags-1 lags-1 lags lags lags-1]); 
else X=mklagsMine([sentiment fyff inflation un lsSpread],[0 0 1 1 0; lags lags lags-1 lags-1 lags]); end;
Xall=[ones(length(X.xl),1) X.xl]; y=consumptionLHS(lags+1:end); Xcnst=ones(size(y));

if (rollStart-rollSpan)<1978; disp('The span of the rolling Window is wider than data available, cutting it'); rollSpan=rollStart-1978; end;
XsentOnly=XsentOnly((rollStart-1978-rollSpan)*12+1:end,:); XCons=XCons((rollStart-1978-rollSpan)*12+1:end,:); 
XnonsentOnly=XnonsentOnly((rollStart-1978-rollSpan)*12+1:end,:);
Xall=Xall((rollStart-1978-rollSpan)*12+1:end,:); Xcnst=Xcnst((rollStart-1978-rollSpan)*12+1:end,:); 
y=y((rollStart-1978-rollSpan)*12+1:end,:); 
[minCr,lagsInd,infoC] = infoCrit(XCons,y,[2,pMax+1],0);

% Some Preliminary Results
if printIndex>0;
    printCutoff=(rollEnd-rollStart-rollSpan)*12+12-hForward-2;
    yPrint=y(rollSpan*12:printCutoff); XsentOnlyPrint=XsentOnly(rollSpan*12:printCutoff,:);
    XnonsentOnlyPrint=XnonsentOnly(rollSpan*12:printCutoff,:); XallPrint=Xall(rollSpan*12:printCutoff,:);
    XConsPrint=XCons(rollSpan*12:printCutoff,:);
	betaSO=XsentOnlyPrint\yPrint; betaC=XConsPrint\yPrint; betaNSO=XnonsentOnlyPrint\yPrint; betaAll=XallPrint\yPrint;
	[junk,junk,nwSO,junk]=olshac(yPrint,XsentOnlyPrint,hForward-1,0,3); 
	[junk,junk,nwC,junk]=olshac(yPrint,XConsPrint,hForward-1,0,3); 
    [junk,junk,nwNSO,junk]=olshac(yPrint,XnonsentOnlyPrint,hForward-1,0,3); 
	[junk,junk,nwA,junk]=olshac(yPrint,XallPrint,hForward-1,0,3); 
	resids=repmat(yPrint,1,4)-[XallPrint*betaAll, XsentOnlyPrint*betaSO, XnonsentOnlyPrint*betaNSO, XConsPrint*betaC]; varRes=var(resids);
    nVars=[size(XallPrint,2) size(XsentOnlyPrint,2) size(XnonsentOnlyPrint,2) size(XConsPrint,2)];
	rsq1=1-var(resids)/var(yPrint); rsq2=1-(var(resids)*length(yPrint)./(length(yPrint)-nVars))/var(yPrint);
    rsq=[rsq1; rsq2];
	regprint(betaAll,[],[],nwA,resids(:,1),1,length(yPrint),rsq(:,1));
    if printIndex>1;
        regprint(betaSO,[],[],nwSO,resids(:,2),1,length(yPrint),rsq(:,2));
        regprint(betaNSO,[],[],nwNSO,resids(:,3),1,length(yPrint),rsq(:,3));
        regprint(betaC,[],[],nwC,resids(:,4),1,length(yPrint),rsq(:,4));
    end;
end;
% We're rollin'
%==========================================================================
rollNumb=(rollEnd-rollStart+1)*12-lags-1-hForward; rollWindow=rollSpan*12;
rollTime=time((rollStart-1978)*12+1:(rollStart-1978)*12+rollNumb);
for i=1:rollNumb; 
   regStart=(fixWindow==1)*i+(fixWindow~=1);
   % Rolling Regressions
   switch fixIndex;
     case 1;
        [rollFAll(i,1),rollEAll(i,1),rollAct(i,1)]=rollFunc(regStart,rollWindow+i,Xall,y);  
        [rollFNonsentOnly(i,1),rollENonsentOnly(i,1),rollAct(i,1)]=rollFunc(regStart,rollWindow+i,XnonsentOnly,y);
        [rollFSentOnly(i,1),rollESentOnly(i,1),rollAct(i,1),const(i,1)]=rollFunc(regStart,rollWindow+i,XsentOnly,y);
        [rollFCons(i,1),rollECons(i,1),rollAct(i,1)]=rollFunc(regStart,rollWindow+i,XCons,y);
     case 2;
        [rollFAll(i,1),rollEAll(i,1),rollAct(i,1),ellIA(:,i)]=rollFunc3(regStart,rollWindow+i,Xall,y,pMax,bicIndex); 
        [rollFSentOnly(i,1),rollESentOnly(i,1),rollAct(i,1),ellIS(:,i)]=rollFunc3(regStart,rollWindow+i,XsentOnly,y,pMax,bicIndex); 
        [rollFNonsentOnly(i,1),rollENonsentOnly(i,1),rollAct(i,1),ellINS(:,i)]=rollFunc3(regStart,rollWindow+i,XnonsentOnly,y,pMax,bicIndex); 
        [rollFCons(i,1),rollECons(i,1),rollAct(i,1),ellIC(:,i)]=rollFunc3(regStart,rollWindow+i,XCons,y,pMax,bicIndex);       
   end;
   if plsIndex  
      [rollFAllP(:,i),rollEAllP(:,i),rollActP(i,1)]=rollFuncPLS2(regStart,rollWindow+i,Xall,y,pMax);
      [rollFSentOnlyP(:,i),rollESentOnlyP(:,i),rollActP(i,1)]=rollFuncPLS2(regStart,rollWindow+i,XsentOnly,y,pMax);
      [rollFNonsentOnlyP(:,i),rollENonsentOnlyP(:,i),rollActP(i,1)]=rollFuncPLS2(regStart,rollWindow+i,XnonsentOnly,y,pMax);
      [rollFConsP(:,i),rollEConsP(:,i),rollActP(i,1)]=rollFuncPLS2(regStart,rollWindow+i,XCons,y,pMax);
   end;
   [rollFcnst(i,1),rollEcnst(i,1),rollAct(i,1)]=rollFunc(regStart,rollWindow+i,Xcnst,y); 
end;

if fixIndex>1;
	mean(ellIA')
	mean(ellIS')
	mean(ellINS')
	mean(ellIC')
end;

if plsIndex; 
	mseSentOnlyP=mean(rollESentOnlyP'.^2); mseConsP=mean(rollEConsP'.^2);
	mseNonsentOnlyP=mean(rollENonsentOnlyP'.^2); mseAllP=mean(rollEAllP'.^2);
	mseP=[min(mseAllP); min(mseSentOnlyP); min(mseNonsentOnlyP); min(mseConsP)];
	[indA,rawIndA]=spitInd(mseAllP,lags+1); indA'
	[indSO,rawIndSOP]=spitInd(mseSentOnlyP,lags+1); indSO'
	[indNS,rawIndNP]=spitInd(mseNonsentOnlyP,lags+1); indNS'
	[indC,rawIndCFP]=spitInd(mseConsP,lags+1); indC'
    foreAP=rollFAllP(rawIndA,:); foreSOP=rollFSentOnlyP(rawIndSOP,:); foreNP=rollFNonsentOnlyP(rawIndNP,:); foreCFP=rollFConsP(rawIndCFP,:);
end;

fMix=.5*rollFSentOnly+.5*rollFAll; 
forecasts=[rollFAll rollFSentOnly rollFNonsentOnly rollFcnst fMix]; 
[lambda,seLambda,fOptim,rAdjC]=fComb(rollAct,forecasts(:,1:end-1),hForward,printIndex); lambda=lambda'; 
eMix=fMix-rollAct; eOptim=fOptim-rollAct;
errors=[rollEAll rollESentOnly rollENonsentOnly rollEcnst eMix eOptim]; errors2=errors.^2; forecasts=[forecasts fOptim];
k=size(forecasts,2); mseSmooth=ma(errors2,smooth); mse=mean(errors2);

if plsIndex; 
	fMixP=.5*foreAP+.5*foreSOP; forecastsP=[foreAP' foreSOP' foreNP' rollFcnst fMixP']; 
    [betaRP,seSP,fOptimP,rAdjP]=fComb(rollAct,forecastsP,hForward,printIndex);
    forecastsP=[forecastsP fOptimP];
    mseP=mean((forecastsP-repmat(rollAct,1,k)).^2);
end;

% mean([errors(1:end-108,:).^2 eMix(1:end-108).^2 eOptim(1:end-108).^2]); %pre and post 94 MSEs
% mean([errors(109:end,:).^2 eMix(109:end).^2 eOptim(109:end).^2]);

% Mincer-Zarnowitz and Diebold-Mariano
[rAdjMZ,fStatMZ,pValMZ] = mzReg(rollAct,forecasts,hForward,printIndex);
[dmSDM,sdDM,pValDM]=dm(errors2,hForward-1)

% Clark-McCracken stuff
lambda= 1/(1+rollNumb/rollWindow); bfc=nan; sds=nan; dmStats=nan; pValDMN=nan;

XallCM=Xall(rollWindow+2:rollWindow+rollNumb+1,:); XsentOnlyCM=XsentOnly(rollWindow+2:rollWindow+rollNumb+1,:);
XnonsentOnlyCM=XnonsentOnly(rollWindow+2:rollWindow+rollNumb+1,:); XcnstCM=Xcnst(rollWindow+2:rollWindow+rollNumb+1,:);
if consIncIndex; nonOrder=[1:lags+1, 2*lags+2:6*lags+1, lags+2:2*lags+1]; else; nonOrder=[1, lags+2:5*lags+1, 2:lags+1]; end;
% be sure to have the regressors in xSmall and xBig in the same order with
% the additional regs in xBig at the end
plotIndex=0; nReps=10000; nN=200;

bwIndex=1;
figure; h2=plot(rollTime, mseSmooth(:,[1 3 4])); axis tight; axis 'auto y';
set(h2,'linewidth',1.5); set(h2(2),'linestyle','--'); set([h2(3)],'linestyle','-'); set(h2(1),'color',cc.darkBlue)
titleString=[sentName ', Smoothed MSEs, smoothing window = ' int2str(smooth)]; title(titleString); axis([-inf inf 0 16]);
if bwIndex; set(h2,'color','k'); set(h2(3),'linestyle',':'); end;
legend('Big','Non Sentiment','Random Walk',0); 

%  Clark-McCracken DM stats
[dmSASCM,junk,pValASCM]=dmCM(errors(:,2),errors(:,1),XsentOnlyCM,XallCM,lambda,hForward-1,plotIndex,nN,nReps); 
[dmSANCM,junk,pValANCM]=dmCM(errors(:,3),errors(:,1),XnonsentOnlyCM,XallCM(:,nonOrder),lambda,hForward-1,plotIndex,nN,nReps); 
[dmSRWCM,junk,pValRWCM]=dmCM(errors(:,4),errors(:,1),XcnstCM,XallCM,lambda,hForward-1,plotIndex,nN,nReps); 
pValMC=[pValASCM; pValANCM;pValRWCM];

[dmSMix,sdMix,pValMix]=dm2(errors(:,5),errors(:,1),hForward-1)
[dmSOpt,sdOpt,pValOpt]=dm2(errors(:,6),errors(:,1),hForward-1)
dmStats=[dmSASCM; dmSANCM; dmSRWCM; dmSMix; dmSOpt];
pValDMN=[pValMix; pValOpt];

disp('Clark-McCracken DM Stats         CVs');
disp([[dmSASCM; dmSANCM; dmSRWCM],[pValASCM; pValANCM; pValRWCM]]); pValsDMCM=[pValASCM; pValANCM; pValRWCM];

% Clark-McCracken t stats (forecast combination)
[bSAS,seSAS,pValSASCM]=fCombCM2(rollAct,forecasts(:,1:2),XsentOnlyCM,XallCM,lambda,hForward,plotIndex,nN,nReps); 
[bSAN,seSAN,pValSANCM]=fCombCM2(rollAct,forecasts(:,[1 3]),XnonsentOnlyCM,XallCM(:,nonOrder),lambda,hForward,plotIndex,nN,nReps); 
[bSARW,seSARW,pValSARWCM]=fCombCM2(rollAct,forecasts(:,[1 4]),XcnstCM,XallCM,lambda,hForward,plotIndex, nN,nReps); 
[bSAMix,seSAMix,pValSAMix]=fComb2(rollAct,forecasts(:,[1 5]),hForward)
[bSAOpt,seSAOpt,pValSAOpt]=fComb2(rollAct,forecasts(:,[1 6]),hForward)

bfc=[bSAS;bSAN;bSARW;bSAMix;bSAOpt]; sds=[seSAS; seSAN; seSARW;seSAMix;seSAOpt]; fcCM=repmat(bfc,1,2)./sds;

disp('Clark-McCracken ENC t stats      CVs');
disp([fcCM(1:3,:),[pValSASCM; pValSANCM; pValSARWCM]]); pValsDMT=[pValASCM; pValANCM; pValRWCM];
