cd D:\users\jslacalek\research\sentiment\web\realTime;
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
close all; clear all; diary([mfilename '.out']); 
disp('==================================================================================');
disp(['Running  ' mfilename '.m                      ' datestr(now)]); format compact; tic; initSeed;
disp('==================================================================================');
data=load('dataQreal.txt');

% Choose parameters
%==========================================================================
rollSpan=6;         %rollSpan: # years for window (at the beginning)
fixWindow=0;        % fixWindow: 1 fixed, otherwise increasing
lags=1;             %# of lags in the forecasting regressions % be careful, with 4 lags it takes about 9 min on Pentium 1.80, with 6 lags several hours
pMax=lags;          % maximum number of lags for information criteria
consIndex=6;        % consumption index: 1 total, 2 durable, 3 nondurable, 4 services, 5 RT RT, 6 RT Fin
sentIndex=3;        % sentiment index: 1 michigan total, 2 michigan expectations, 3 CB total, 4 CB expectations                         
smooth=10;          % window for the MA filter 
hForward=1;         % # of periods ahead to calculate forecasts (ie 12: forecasting consumption growth over next year
consIncIndex=1;     % include lagged consumption growth in the models?
fixIndex=1;         % determines the number of lags: 1 equal to lags, 2 using info criteria
bicIndex=0;         % BIC or AIC ?
plsIndex=0;         % predictiveleast squares?
printIndex=2;       % print out regression diagnostics for forecasts: 1 prints just one regression; 2 prints four intro regressions, 3 prints everything
bootIndex=0;        % do you want to run a bootstrap for D-M test?
bootReps=5e3;       % # of bootstrap repetitions
freq=4;

% Regresion Matrices
%==========================================================================
transformDataQR; %call script to rename and 'stationarize' data
rollEnd=2002; rollStart=1970+rollSpan; %rollStart=1987; % uncomment to pick rollStart by yourself
fprintf('Time Frame:  %4.0f  %4.0f      Lags: %2.0f      %2.0f Period(s) ahead\n', rollStart,rollEnd,lags,hForward);

switch consIndex; 
    case 1; consumption=totalCons; consumptionLHS=totalConsLHS; seriesNames='Total Consumption      '; 
    case 2; consumption=durableCons; consumptionLHS=durableConsLHS; seriesNames='Durable Consumption        ';
    case 3; consumption=nondurableCons; consumptionLHS=nondurableConsLHS; seriesNames='Nondurable Consumption       '; 
    case 4; consumption=serviceCons; consumptionLHS=serviceConsLHS; seriesNames='Services       '; 
    case 5; consumption=rtConsG; consumptionLHS=rtConsG; seriesNames='Real-Time (Both)   '; 
    case 6; consumption=rtConsG; consumptionLHS=totalConsLHS; seriesNames='Real-Time (RHS)   '; 
end;
switch sentIndex; 
    case 1; sentiment=michSent; sentName='Total Michigan'; seriesNames=[seriesNames '   ' sentName];
    case 2; sentiment=michExp; sentName='Michigan Expectations'; seriesNames=[seriesNames '   ' sentName];
    case 3; sentiment=cbSent; sentName='Total CB'; seriesNames=[seriesNames '   ' sentName]; 
    case 4; sentiment=cbExp; sentName='CB Expectations'; seriesNames=[seriesNames '   ' sentName]; 
end;
disp(seriesNames);

if consIncIndex==1; X=mklagsMine([consumption sentiment],[1 0; lags lags-1]); else X=mklagsMine([sentiment],[0; lags-1]); end;    
XsentOnly=[ones(length(X.xl),1) X.xl];
X=mklagsMine([consumption],[1; lags]); XCons=[ones(length(X.xl),1) X.xl];
if consIncIndex==1; X=mklagsMine([consumption fyff inflation un lsSpread],[1 0 1 1 0; lags lags-1 lags lags lags-1]); 
else X=mklagsMine([fyff inflation un lsSpread],[0 1 1 0; lags-1 lags lags lags-1]); end;
XnonsentOnly=[ones(length(X.xl),1) X.xl];
if consIncIndex==1; X=mklagsMine([consumption sentiment fyff inflation un lsSpread],[1 0 0 1 1 0; lags lags-1 lags-1 lags lags lags-1]); 
else X=mklagsMine([sentiment fyff inflation un lsSpread],[0 0 1 1 0; lags lags lags-1 lags-1 lags]); end;
Xall=[ones(length(X.xl),1) X.xl]; y=consumptionLHS(lags+1:end); Xcnst=ones(size(y));


if (rollStart-rollSpan)<1970; disp('The span of the rolling Window is wider than data available, cutting it'); rollSpan=rollStart-1970; end;
XsentOnly=XsentOnly((rollStart-1970-rollSpan)*freq+1:end,:); XCons=XCons((rollStart-1970-rollSpan)*freq+1:end,:); 
XnonsentOnly=XnonsentOnly((rollStart-1970-rollSpan)*freq+1:end,:); 
Xall=Xall((rollStart-1970-rollSpan)*freq+1:end,:); Xcnst=Xcnst((rollStart-1970-rollSpan)*freq+1:end,:); 
y=y((rollStart-1970-rollSpan)*freq+1:end,:); 
[minCr,lagsInd,infoC] = infoCrit(XCons,y,[2,pMax+1],0);

% Some Preliminary Results
if printIndex>0;
    printCutoff=length(XsentOnly); %(rollEnd-rollStart-rollSpan)*freq+freq-hForward-2;
    yPrint=y(rollSpan*freq:printCutoff); XsentOnlyPrint=XsentOnly(rollSpan*freq:printCutoff,:);
    XnonsentOnlyPrint=XnonsentOnly(rollSpan*freq:printCutoff,:); XallPrint=Xall(rollSpan*freq:printCutoff,:);
    XConsPrint=XCons(rollSpan*freq:printCutoff,:);
	betaSO=XsentOnlyPrint\yPrint; betaC=XConsPrint\yPrint; betaNSO=XnonsentOnlyPrint\yPrint; betaAll=XallPrint\yPrint;
	[junk,junk,nwSO,junk]=olshac(yPrint,XsentOnlyPrint,hForward-1,0,3); 
	[junk,junk,nwC,junk]=olshac(yPrint,XConsPrint,hForward-1,0,3); 
    [junk,junk,nwNSO,junk]=olshac(yPrint,XnonsentOnlyPrint,hForward-1,0,3); 
	[junk,junk,nwA,junk]=olshac(yPrint,XallPrint,hForward-1,0,3); 
	resids=repmat(yPrint,1,4)-[XallPrint*betaAll, XsentOnlyPrint*betaSO, XnonsentOnlyPrint*betaNSO, XConsPrint*betaC]; varRes=var(resids);
    nVars=[size(XallPrint,2) size(XsentOnlyPrint,2) size(XnonsentOnlyPrint,2) size(XConsPrint,2)];
	rsq1=1-var(resids)/var(yPrint); rsq2=1-(var(resids)*length(yPrint)./(length(yPrint)-nVars))/var(yPrint);
    rsq=[rsq1; rsq2];
	regprint(betaAll,[],[],nwA,resids(:,1),1,length(yPrint),rsq(:,1));
    if printIndex>1;
        regprint(betaSO,[],[],nwSO,resids(:,2),1,length(yPrint),rsq(:,2));
        regprint(betaNSO,[],[],nwNSO,resids(:,3),1,length(yPrint),rsq(:,3));
        regprint(betaC,[],[],nwC,resids(:,4),1,length(yPrint),rsq(:,4));
    end;
end;
% We're rollin'
%==========================================================================
rollNumb=(rollEnd-rollStart+1)*freq-lags-1-hForward; rollWindow=rollSpan*freq;
rollTime=time((rollStart-1970)*freq+1:(rollStart-1970)*freq+rollNumb);
for i=1:rollNumb; 
   regStart=(fixWindow==1)*i+(fixWindow~=1);
   % Rolling Regressions
   switch fixIndex;
     case 1;
        [rollFAll(i,1),rollEAll(i,1),rollAct(i,1)]=rollFunc(regStart,rollWindow+i,Xall,y);  
        [rollFNonsentOnly(i,1),rollENonsentOnly(i,1),rollAct(i,1)]=rollFunc(regStart,rollWindow+i,XnonsentOnly,y);
        [rollFSentOnly(i,1),rollESentOnly(i,1),rollAct(i,1),const(i,1)]=rollFunc(regStart,rollWindow+i,XsentOnly,y);
        [rollFCons(i,1),rollECons(i,1),rollAct(i,1)]=rollFunc(regStart,rollWindow+i,XCons,y);
     case 2;
        [rollFAll(i,1),rollEAll(i,1),rollAct(i,1),ellIA(:,i)]=rollFunc3(regStart,rollWindow+i,Xall,y,pMax,bicIndex); 
        [rollFSentOnly(i,1),rollESentOnly(i,1),rollAct(i,1),ellIS(:,i)]=rollFunc3(regStart,rollWindow+i,XsentOnly,y,pMax,bicIndex); 
        [rollFNonsentOnly(i,1),rollENonsentOnly(i,1),rollAct(i,1),ellINS(:,i)]=rollFunc3(regStart,rollWindow+i,XnonsentOnly,y,pMax,bicIndex); 
        [rollFCons(i,1),rollECons(i,1),rollAct(i,1),ellIC(:,i)]=rollFunc3(regStart,rollWindow+i,XCons,y,pMax,bicIndex);       
   end;
   if plsIndex  
      [rollFAllP(:,i),rollEAllP(:,i),rollActP(i,1)]=rollFuncPLS2(regStart,rollWindow+i,Xall,y,pMax);
      [rollFSentOnlyP(:,i),rollESentOnlyP(:,i),rollActP(i,1)]=rollFuncPLS2(regStart,rollWindow+i,XsentOnly,y,pMax);
      [rollFNonsentOnlyP(:,i),rollENonsentOnlyP(:,i),rollActP(i,1)]=rollFuncPLS2(regStart,rollWindow+i,XnonsentOnly,y,pMax);
      [rollFConsP(:,i),rollEConsP(:,i),rollActP(i,1)]=rollFuncPLS2(regStart,rollWindow+i,XCons,y,pMax);
   end;
   [rollFcnst(i,1),rollEcnst(i,1),rollAct(i,1)]=rollFunc(regStart,rollWindow+i,Xcnst,y); 
end;

if fixIndex>1;
	mean(ellIA')
	mean(ellIS')
	mean(ellINS')
	mean(ellIC')
end;

if plsIndex; 
	mseSentOnlyP=mean(rollESentOnlyP'.^2); mseConsP=mean(rollEConsP'.^2);
	mseNonsentOnlyP=mean(rollENonsentOnlyP'.^2); mseAllP=mean(rollEAllP'.^2);
	mseP=[min(mseAllP); min(mseSentOnlyP); min(mseNonsentOnlyP); min(mseConsP)];
	[indA,rawIndA]=spitInd(mseAllP,lags+1); indA'
	[indSO,rawIndSOP]=spitInd(mseSentOnlyP,lags+1); indSO'
	[indNS,rawIndNP]=spitInd(mseNonsentOnlyP,lags+1); indNS'
	[indC,rawIndCFP]=spitInd(mseConsP,lags+1); indC'
    foreAP=rollFAllP(rawIndA,:); foreSOP=rollFSentOnlyP(rawIndSOP,:); foreNP=rollFNonsentOnlyP(rawIndNP,:); foreCFP=rollFConsP(rawIndCFP,:);
end;

fMix=.5*rollFSentOnly+.5*rollFAll; 
forecasts=[rollFAll rollFSentOnly rollFNonsentOnly rollFcnst fMix]; 
[betaRC,seSC,fOptim,rAdjC]=fComb1(rollAct,forecasts(:,[1 3 4]),hForward,printIndex);
eMix=fMix-rollAct; eOptim=fOptim-rollAct;
errors=[rollEAll rollESentOnly rollENonsentOnly rollEcnst eMix eOptim]; errors2=errors.^2; forecasts=[forecasts fOptim];
k=size(forecasts,2); mseSmooth=ma(errors2,smooth); mse=mean(errors2)

% Forecast and MSE Figures
%==========================================================================
cc=customColors;
%figure; cc=customColors;
%h1=plot(rollTime,[rollAct forecasts(:,[1 3 4])]); axis tight; axis 'auto y'; 
% set(h1(2),'linestyle','--','marker','.'); set(h1(3),'linestyle','-.'); set(h1(4),'linestyle',':'); set(h1(5),'marker','.');
% set(h1(1),'linewidth',1.5);
%legend('Actual','Big','Sentiment','Non Sentiment','Random Walk','Big'); title('Consumption Forecasts');
%hold on; plot(rollTime,zeros(length(rollAct),1)); hold off;


figure; h2=plot(rollTime, mseSmooth(:,1:end-2)); legend('Big','Sentiment','Non Sentiment','Random Walk'); axis tight; axis 'auto y';
set(h2,'linewidth',1.5); set(h2(2),'linestyle','--'); set([h2(1)],'linestyle','-.'); set(h2(4),'marker','.');
titleString=['Smoothed MSEs, smoothing window = ' int2str(smooth)]; title(titleString); 

bwIndex=0;
f2=figure; h2=plot(rollTime, mseSmooth(:,[1 3 4])); axis tight; axis 'auto y';
set(h2,'linewidth',2); set([h2(3)],'linestyle','-'); set(h2(2),'color',cc.darkBlue,'linestyle','--');
set(h2(1),'color',cc.red); set(h2(3),'color',cc.grey);
titleString=[sentName ', Smoothed MSEs, smoothing window = ' int2str(smooth)]; title(titleString); axis([-inf inf 0 16]);
if bwIndex; set(h2,'color','k'); set(h2(3),'linestyle',':'); end;
legend('Big','Non Sentiment','Random Walk',1);
print(f2,'-depsc',strcat('fSentRT',int2str(sentIndex),'.eps'));

% Bootstrap
%==========================================================================
if bootIndex~=0;
    bDM=repmat(nan,bootReps,k); bMSE=bDM;
    for i=1:bootReps;
        [bDM(i,:),bMSE(i,:)]=dmBoot(errors2,hForward-1);
    end; 

	bMSE=sort(bMSE); bDM=sort(bDM);
% 	bins=30; 
% 	figure; subplot(2,2,1); hist(bMSE(:,1),bins); title('Sentiment Only'); subplot(2,2,2); hist(bMSE(:,2),bins); 
% 	title('No Sentiment'); subplot(2,2,3); hist(bMSE(:,3),bins); title('All');
% 	figure; subplot(2,2,1); hist(bDM(:,1),bins); title('Sentiment Only'); subplot(2,2,2); hist(bDM(:,2),bins); 
% 	title('No Sentiment'); subplot(2,2,3); hist(bDM(:,3),bins); title('All');
	
	if printIndex>2;
		disp('95% Bootstrap Confidence Intervals and Medians: MSEs:'); disp('Big; Sentiment; No Sentiment; Consumption; Mixed; Optimal');
		disp([bMSE(down,:); bMSE(up,:); bMSE(mid,:)]);
		disp('95% Asymptotic Confidence Intervals and Actual Values: MSEs:'); disp('Big; Sentiment; No Sentiment; Consumption; Mixed; Optimal');
		sdMSE=sqrt(var(errors2)/rollNumb); msed=mse-1.96*sdMSE; mseu=mse+1.96*sdMSE; 
		disp([msed; mseu; mse]);
		
        dmStats=repmat(errors2(:,k),1,k)-errors2;
		disp('95% Bootstrap Confidence Intervals and Medians: Diebold-Mariano Statistics:'); disp('Sent-No Sent; Sent-All; No Sent-All');
		disp([bDM(down,:); bDM(up,:); bDM(mid,:)]);
		disp('95% Asymptotic Confidence Intervals and Actual Values: Diebold-Mariano Statistics:'); disp('Sent-No Sent; Sent-All; No Sent-All');
		sdDM=sqrt(var(mean(dmStats))/rollNumb); dmd=mean(dmStats)-1.96*sdDM; dmu=mean(dmStats)+1.96*sdDM; 
		disp([dmd; dmu; mean(dmStats)]);
	end;
end;

if plsIndex; 
	fMixP=.5*foreAP+.5*foreSOP; forecastsP=[foreAP' foreSOP' foreNP' rollFcnst fMixP']; 
    [betaRP,seSP,fOptimP,rAdjP]=fComb1(rollAct,forecastsP,hForward,printIndex);
    forecastsP=[forecastsP fOptimP];
    mseP=mean((forecastsP-repmat(rollAct,1,k)).^2)
end;

if ~isempty(errors(1:end-108,:));
	mean([errors(1:end-108,:).^2 eMix(1:end-108).^2 eOptim(1:end-108).^2]); %pre and post 94 MSEs
	mean([errors(109:end,:).^2 eMix(109:end).^2 eOptim(109:end).^2]);
end;

% Mincer-Zarnowitz and Diebold-Mariano
[rAdjMZ,fStatMZ,pValMZ] = mzReg(rollAct,[forecasts fMix fOptim],hForward,printIndex);
[dmSDM,sdDM,pValDM]=dm(errors2,hForward-1);

% Clark-McCracken stuff
lambda= 1/(1+rollNumb/rollWindow);

XallCM=Xall(rollWindow+2:rollWindow+rollNumb+1,:); XsentOnlyCM=XsentOnly(rollWindow+2:rollWindow+rollNumb+1,:);
XnonsentOnlyCM=XnonsentOnly(rollWindow+2:rollWindow+rollNumb+1,:); XcnstCM=Xcnst(rollWindow+2:rollWindow+rollNumb+1,:);
if consIncIndex; nonOrder=[1:lags+1, 2*lags+2:6*lags+1, lags+2:2*lags+1]; else; nonOrder=[1, lags+2:5*lags+1, 2:lags+1]; end;
% be sure to have the regressors in xSmall and xBig in the same order with
% the additional regs in xBig at the end
plotIndex=0;

[dmSASCM,junk,pValASCM]=dmCM(errors(:,2),errors(:,1),XsentOnlyCM,XallCM,lambda,hForward-1,plotIndex); 
[dmSANCM,junk,pValANCM]=dmCM(errors(:,3),errors(:,1),XnonsentOnlyCM,XallCM(:,nonOrder),lambda,hForward-1,plotIndex); 
[dmSRWCM,junk,pValRWCM]=dmCM(errors(:,4),errors(:,1),XcnstCM,XallCM,lambda,hForward-1,plotIndex); 
[dmSMix,sdMix,pValMix]=dm2(errors(:,5),errors(:,1),hForward-1)
[dmSOpt,sdOpt,pValOpt]=dm2(errors(:,6),errors(:,1),hForward-1)
dmStats2=[dmSASCM; dmSANCM; dmSRWCM; dmSMix; dmSOpt];
pValDMN=[pValMix; pValOpt];

[bSAS,seSAS,pValSASCM]=fCombCM2(rollAct,forecasts(:,1:2),XsentOnlyCM,XallCM,lambda,hForward,plotIndex);
[bSAN,seSAN,pValSANCM]=fCombCM2(rollAct,forecasts(:,[1 3]),XnonsentOnlyCM,XallCM(:,nonOrder),lambda,hForward,plotIndex);
[bSARW,seSARW,pValSARWCM]=fCombCM2(rollAct,forecasts(:,[1 4]),XcnstCM,XallCM,lambda,hForward,plotIndex);
[bSAMix,seSAMix,pValSAMix]=fComb2(rollAct,forecasts(:,[1 5]),hForward)
[bSAOpt,seSAOpt,pValSAOpt]=fComb2(rollAct,forecasts(:,[1 6]),hForward)

bfc=[bSAS;bSAN;bSARW]; sds=[seSAS; seSAN; seSARW];
fcCM=repmat(bfc,1,2)./sds;

disp('Clark-McCracken DM Stats         CVs');
disp([[dmSASCM; dmSANCM; dmSRWCM],[pValASCM; pValANCM; pValRWCM]]);
disp('Clark-McCracken ENC t stats      CVs');
disp([fcCM,[pValSASCM; pValSANCM; pValSARWCM]]);

% save .\clarkMcCracken\vardata consumptionLHS consumption sentiment fyff inflation un lsSpread rollAct forecasts errors;

t=toc; mints=floor(toc/60); secs=toc-60*mints;
fprintf('Time: %4.0f minutes, %4.2f seconds\n',mints,secs); diary off;

